@@ -1,18 +1,26 @@
 # include "rgdal.h"
 
-SEXP make_Polygonlist(SEXP iG) {
+SEXP make_Polygonlist(SEXP iG, SEXP iGc) {
 
-    SEXP res, jG;
+    SEXP res, jG, jGc;
     int n, i, pc=0;
 
     n = length(iG);
 
     PROTECT(res = NEW_LIST(n)); pc++;
+    PROTECT(jGc = NEW_INTEGER(0)); pc++;
 
     for (i=0; i<n; i++) {
 
         jG = VECTOR_ELT(iG, i);
-        SET_VECTOR_ELT(res, i, make_Polygon(jG));
+// hole setting based on comments by default (OGC SFS order)
+// but by ring order if comment NULL 121019
+        if (iGc == R_NilValue) {
+            SET_VECTOR_ELT(res, i, make_Polygon(jG, R_NilValue));
+        } else {
+            INTEGER_POINTER(jGc)[0] = INTEGER_POINTER(iGc)[i];
+            SET_VECTOR_ELT(res, i, make_Polygon(jG, jGc));
+        }
     }
 
     UNPROTECT(pc);
@@ -20,7 +28,7 @@ SEXP make_Polygonlist(SEXP iG) {
 
 }
 
-SEXP make_Polygon(SEXP jG) {
+SEXP make_Polygon(SEXP jG, SEXP jGc) {
 
     SEXP res, coords, dim, dimnames, n, ihole;
     int i, nn, pc=0, copy1=FALSE;
@@ -55,10 +63,12 @@ SEXP make_Polygon(SEXP jG) {
         }
     }
 
+// hole setting based on comments by default (OGC SFS order)
+// but by ring order if comment NULL 121019
     PROTECT(ihole = NEW_INTEGER(1)); pc++;
-//    INTEGER_POINTER(ihole)[0] = NA_INTEGER;
-    INTEGER_POINTER(ihole)[0] = INTEGER_POINTER(getAttrib(jG,
-        install("hole")))[0];
+    INTEGER_POINTER(ihole)[0] = 0L;
+    if (jGc == R_NilValue) INTEGER_POINTER(ihole)[0] = NA_INTEGER;
+    else if (INTEGER_POINTER(jGc)[0] != 0L) INTEGER_POINTER(ihole)[0] = 1L;
     PROTECT(n = NEW_INTEGER(1)); pc++;
     if (copy1) INTEGER_POINTER(n)[0] = nn+1;
     else INTEGER_POINTER(n)[0] = nn;
